@@ -28,8 +28,10 @@
 )]
 
 use dirs_next::home_dir;
-use sn_launch_tool::run_with;
+use eyre::{eyre, Result, WrapErr as _};
+use sn_launch_tool::Launch;
 use std::{
+    io,
     path::PathBuf,
     process::{Command, Stdio},
 };
@@ -37,6 +39,7 @@ use structopt::StructOpt;
 use tokio::fs::{create_dir_all, remove_dir_all};
 use tokio::time::{sleep, Duration};
 use tracing::{debug, info};
+use tracing_subscriber::EnvFilter;
 
 #[cfg(not(target_os = "windows"))]
 const SAFE_NODE_EXECUTABLE: &str = "sn_node";
@@ -44,10 +47,10 @@ const SAFE_NODE_EXECUTABLE: &str = "sn_node";
 #[cfg(target_os = "windows")]
 const SAFE_NODE_EXECUTABLE: &str = "sn_node.exe";
 
+const BASE_TRACING_DIRECTIVES: &str = "testnet=info,sn_launch_tool=debug";
 const NODES_DIR: &str = "local-test-network";
-const INTERVAL: &str = "2";
-const RUST_LOG: &str = "RUST_LOG";
-const NODE_COUNT: &str = "33";
+const INTERVAL: Duration = Duration::from_secs(2);
+const DEFAULT_NODE_COUNT: u32 = 33;
 
 #[derive(Debug, StructOpt)]
 #[structopt(name = "testnet")]
@@ -58,12 +61,21 @@ struct Cmd {
 }
 
 #[tokio::main]
-async fn main() -> Result<(), String> {
+async fn main() -> Result<()> {
+    color_eyre::install()?;
+    init_tracing()?;
+
     let path = std::path::Path::new("nodes");
-    remove_dir_all(&path).await.unwrap_or(()); // Delete nodes directory if it exists;
+    remove_dir_all(&path)
+        .await
+        .or_else(|error| match error.kind() {
+            io::ErrorKind::NotFound => Ok(()),
+            _ => Err(error),
+        })
+        .wrap_err("Failed to remove existing nodes directory")?;
     create_dir_all(&path)
         .await
-        .expect("Cannot create nodes directory");
+        .wrap_err("Cannot create nodes directory")?;
 
     let mut args = vec!["build", "--release"];
 
@@ -77,7 +89,8 @@ async fn main() -> Result<(), String> {
         args.push("test-utils");
     }
 
-    println!("Building current sn_node");
+    info!("Building current sn_node");
+    debug!("Building current sn_node with args: {:?}", args);
     let _child = Command::new("cargo")
         .args(args.clone())
         // .env("RUST_LOG", "debug")
@@ -85,36 +98,17 @@ async fn main() -> Result<(), String> {
         .stdout(Stdio::inherit())
         .stderr(Stdio::inherit())
         .output()
-        .map_err(|err| {
-            format!(
-                "Failed to run build command with args '{:?}': {}",
-                args, err
-            )
-        })?;
+        .wrap_err_with(|| format!("Failed to run build command with args: {:?}", args))?;
 
-    println!("sn_node built successfully");
+    info!("sn_node built successfully");
 
     run_network().await?;
 
     Ok(())
 }
 
-fn get_node_bin_path(node_path: Option<PathBuf>) -> Result<PathBuf, String> {
-    match node_path {
-        Some(p) => Ok(p),
-        None => {
-            let mut home_dirs =
-                home_dir().ok_or_else(|| "Failed to obtain user's home path".to_string())?;
-
-            home_dirs.push(".safe");
-            home_dirs.push("node");
-            Ok(home_dirs)
-        }
-    }
-}
-
 /// Uses SNLT to create a local network of nodes
-pub async fn run_network() -> Result<(), String> {
+pub async fn run_network() -> Result<()> {
     let args = Cmd::from_args();
     let adding_nodes = args.add_nodes_to_existing_network;
 
@@ -129,26 +123,23 @@ pub async fn run_network() -> Result<(), String> {
     let node_log_dir = base_log_dir.join(NODES_DIR);
     if !node_log_dir.exists() {
         debug!("Creating '{}' folder", node_log_dir.display());
-        create_dir_all(node_log_dir.clone()).await.map_err(|err| {
-            format!(
-                "Couldn't create target path to store nodes' generated data: {}",
-                err
-            )
-        })?;
+        create_dir_all(node_log_dir.clone())
+            .await
+            .wrap_err("Couldn't create target path to store nodes' generated data")?;
     }
     let arg_node_log_dir = node_log_dir.display().to_string();
     info!("Storing nodes' generated data at {}", arg_node_log_dir);
 
     // Let's create an args array to pass to the network launcher tool
+    let interval_str = INTERVAL.as_secs().to_string();
     let mut sn_launch_tool_args = vec![
         "sn_launch_tool",
-        "-v",
         "--node-path",
         &arg_node_path,
         "--nodes-dir",
         &arg_node_log_dir,
         "--interval",
-        INTERVAL,
+        &interval_str,
         "--local",
     ];
 
@@ -158,20 +149,22 @@ pub async fn run_network() -> Result<(), String> {
 
     // If RUST_LOG was set we pass it down to the launch tool
     // so it's set for each of the nodes logs as well.
-    let rust_log = std::env::var(RUST_LOG).unwrap_or_else(|_| "".to_string());
+    let rust_log =
+        std::env::var(EnvFilter::DEFAULT_ENV).unwrap_or_else(|_| "safe_network=info".to_string());
     if !rust_log.is_empty() {
         sn_launch_tool_args.push("--rust-log");
         sn_launch_tool_args.push(&rust_log);
     }
 
-    let interval_as_int = &INTERVAL
-        .parse::<u64>()
-        .map_err(|_| String::from("Error parsing Interval argument"))?;
-
-    let node_count = std::env::var("NODE_COUNT").unwrap_or_else(|_| NODE_COUNT.to_string());
-    let node_count_as_int = node_count
-        .parse::<u64>()
-        .map_err(|_| String::from("Error parsing Node Count argument"))?;
+    let node_count = std::env::var("NODE_COUNT")
+        .map_or_else(
+            |error| match error {
+                std::env::VarError::NotPresent => Ok(DEFAULT_NODE_COUNT),
+                _ => Err(eyre!(error)),
+            },
+            |node_count| Ok(node_count.parse()?),
+        )
+        .wrap_err("Invalid value for NODE_COUNT")?;
 
     debug!(
         "Running network launch tool with args: {:?}",
@@ -180,12 +173,49 @@ pub async fn run_network() -> Result<(), String> {
 
     // We can now call the tool with the args
     info!("Launching local Safe network...");
-    run_with(Some(&sn_launch_tool_args))?;
+    Launch::from_iter_safe(&sn_launch_tool_args)?.run()?;
 
     // leave a longer interval with more nodes to allow for splits if using split amounts
-    let interval_duration = Duration::from_secs(interval_as_int * node_count_as_int);
+    let interval_duration = INTERVAL * node_count;
 
     sleep(interval_duration).await;
 
     Ok(())
 }
+
+fn init_tracing() -> Result<()> {
+    let mut filter = EnvFilter::try_new(BASE_TRACING_DIRECTIVES)
+        .wrap_err("BUG: hard-coded tracing directives are invalid")?;
+
+    let extra_directives = std::env::var(EnvFilter::DEFAULT_ENV)
+        .map_or_else(
+            |error| match error {
+                std::env::VarError::NotPresent => Ok(None),
+                std::env::VarError::NotUnicode(_) => Err(eyre!(error)),
+            },
+            |filter| Ok(Some(EnvFilter::try_new(filter)?)),
+        )
+        .wrap_err_with(|| format!("Invalid value for {}", EnvFilter::DEFAULT_ENV))?;
+
+    if let Some(extra_directives) = extra_directives {
+        for directive in extra_directives.to_string().split(',') {
+            filter = filter.add_directive(
+                directive
+                    .parse()
+                    .expect("BUG: invalid directive in parsed EnvFilter"),
+            );
+        }
+    }
+
+    tracing_subscriber::fmt().with_env_filter(filter).init();
+
+    Ok(())
+}
+
+fn get_node_bin_path(node_path: Option<PathBuf>) -> Result<PathBuf> {
+    node_path.ok_or(()).or_else(|()| {
+        let mut bin_path = home_dir().ok_or_else(|| eyre!("Failed to obtain user's home path"))?;
+        bin_path.push(".safe/node");
+        Ok(bin_path)
+    })
+}
