@@ -10,7 +10,7 @@ use crate::messaging::WireMsg;
 use crate::routing::error::{Error, Result};
 use bytes::Bytes;
 use futures::stream::{FuturesUnordered, StreamExt};
-use qp2p::{Endpoint, QuicP2p};
+use qp2p::Endpoint;
 use std::net::SocketAddr;
 use tokio::{sync::mpsc, task};
 use xor_name::XorName;
@@ -18,8 +18,7 @@ use xor_name::XorName;
 // Communication component of the node to interact with other nodes.
 #[derive(Clone)]
 pub(crate) struct Comm {
-    quic_p2p: QuicP2p,
-    endpoint: Endpoint,
+    endpoint: Endpoint<XorName>,
 }
 
 impl Drop for Comm {
@@ -31,20 +30,16 @@ impl Drop for Comm {
 
 impl Comm {
     pub(crate) async fn new(
+        local_addr: SocketAddr,
         transport_config: qp2p::Config,
         event_tx: mpsc::Sender<ConnectionEvent>,
     ) -> Result<Self> {
-        let quic_p2p = QuicP2p::with_config(Some(transport_config), &[], true)
-            .map_err(|err| Error::InvalidConfig { err })?;
-
         // Don't bootstrap, just create an endpoint to listen to
         // the incoming messages from other nodes.
         // This also returns the a channel where we can listen for
         // disconnection events.
-        let (endpoint, _incoming_connections, incoming_messages, disconnections) = quic_p2p
-            .new_endpoint()
-            .await
-            .map_err(|err| Error::CannotConnectEndpoint { err })?;
+        let (endpoint, _incoming_connections, incoming_messages, disconnections, _) =
+            Endpoint::new(local_addr, Default::default(), transport_config).await?;
 
         let _ = task::spawn(handle_incoming_messages(
             incoming_messages,
@@ -56,23 +51,20 @@ impl Comm {
             event_tx.clone(),
         ));
 
-        Ok(Self { quic_p2p, endpoint })
+        Ok(Self { endpoint })
     }
 
     pub(crate) async fn bootstrap(
+        local_addr: SocketAddr,
+        bootstrap_nodes: &[SocketAddr],
         transport_config: qp2p::Config,
         event_tx: mpsc::Sender<ConnectionEvent>,
     ) -> Result<(Self, SocketAddr)> {
-        let quic_p2p = QuicP2p::with_config(Some(transport_config), &[], true)
-            .map_err(|err| Error::InvalidConfig { err })?;
-
         // Bootstrap to the network returning the connection to a node.
         // We can use the returned channels to listen for incoming messages and disconnection events
         let (endpoint, _incoming_connections, incoming_messages, disconnections, bootstrap_addr) =
-            quic_p2p
-                .bootstrap()
-                .await
-                .map_err(|err| Error::CannotConnectEndpoint { err })?;
+            Endpoint::new(local_addr, bootstrap_nodes, transport_config).await?;
+        let bootstrap_addr = bootstrap_addr.ok_or(Error::BootstrapFailed)?;
 
         let _ = task::spawn(handle_incoming_messages(
             incoming_messages,
@@ -84,11 +76,21 @@ impl Comm {
             event_tx.clone(),
         ));
 
-        Ok((Self { quic_p2p, endpoint }, bootstrap_addr))
+        Ok((Self { endpoint }, bootstrap_addr))
     }
 
     pub(crate) fn our_connection_info(&self) -> SocketAddr {
-        self.endpoint.socket_addr()
+        self.endpoint.public_addr()
+    }
+
+    /// Get the connection ID (XorName) of an existing connection with the provided socket address
+    pub(crate) async fn get_connection_id(&self, addr: &SocketAddr) -> Option<XorName> {
+        self.endpoint.get_connection_id(addr).await
+    }
+
+    /// Get the SocketAddr of a connection using the connection ID (XorName)
+    pub(crate) async fn get_socket_addr_by_id(&self, xorname: &XorName) -> Option<SocketAddr> {
+        self.endpoint.get_socket_addr_by_id(xorname).await
     }
 
     /// Sends a message on an existing connection. If no such connection exists, returns an error.
@@ -101,12 +103,14 @@ impl Comm {
             wire_msg.set_dst_xorname(*name);
             let bytes = wire_msg.serialize()?;
 
+            let priority = wire_msg.msg_kind().priority();
+
             self.endpoint
-                .try_send_message(bytes, addr)
+                .try_send_message(bytes, addr, priority)
                 .await
                 .map_err(|err| {
                     error!(
-                        "Sending message (msg_id: {}) to {:?} (name {:?}) failed with {}",
+                        "Sending message (msg_id: {}) to {:?} (name {:?}) failed with {:?}",
                         wire_msg.msg_id(),
                         addr,
                         name,
@@ -122,25 +126,19 @@ impl Comm {
     /// Tests whether the peer is reachable.
     pub(crate) async fn is_reachable(&self, peer: &SocketAddr) -> Result<(), Error> {
         let qp2p_config = qp2p::Config {
-            local_ip: Some(self.endpoint.local_addr().ip()),
-            local_port: Some(0),
             forward_port: false,
             ..Default::default()
         };
 
-        let qp2p = QuicP2p::with_config(Some(qp2p_config), &[], false)
-            .map_err(|err| Error::InvalidConfig { err })?;
-        let (connectivity_endpoint, _, _, _) = qp2p
-            .new_endpoint()
-            .await
-            .map_err(|err| Error::CannotConnectEndpoint { err })?;
+        let (connectivity_endpoint, _, _) =
+            Endpoint::<XorName>::new_client((self.endpoint.local_addr().ip(), 0), qp2p_config)?;
 
         let result = connectivity_endpoint
             .is_reachable(peer)
             .await
             .map_err(|err| {
-                info!("Peer {} is NOT externally reachable: {}", peer, err);
-                Error::AddressNotReachable { err }
+                info!("Peer {} is NOT externally reachable: {:?}", peer, err);
+                err.into()
             })
             .map(|()| {
                 info!("Peer {} is externally reachable.", peer);
@@ -164,8 +162,10 @@ impl Comm {
         delivery_group_size: usize,
         wire_msg: WireMsg,
     ) -> Result<SendStatus> {
+        let msg_id = wire_msg.msg_id();
         trace!(
-            "Sending message to {} of {:?}",
+            "Sending message (msg_id: {:?}) to {} of {:?}",
+            msg_id,
             delivery_group_size,
             recipients
         );
@@ -185,29 +185,31 @@ impl Comm {
         }
 
         let msg_bytes = wire_msg.serialize().map_err(Error::Messaging)?;
+        let priority = wire_msg.msg_kind().priority();
 
         // Run all the sends concurrently (using `FuturesUnordered`). If any of them fails, pick
         // the next recipient and try to send to them. Proceed until the needed number of sends
         // succeeds or if there are no more recipients to pick.
         let send = |recipient: (XorName, SocketAddr), msg_bytes: Bytes| async move {
             trace!(
-                "Sending message ({} bytes) to {} of delivery group size {}",
+                "Sending message ({} bytes, msg_id: {:?}) to {} of delivery group size {}",
                 msg_bytes.len(),
+                msg_id,
                 recipient.1,
                 delivery_group_size,
             );
 
             let result = self
                 .endpoint
-                .send_message(msg_bytes, &recipient.1)
+                .send_message(msg_bytes, &recipient.1, priority)
                 .await
                 .map_err(|err| match err {
-                    qp2p::Error::Connection(qp2p::ConnectionError::LocallyClosed) => {
-                        Error::ConnectionClosed
-                    }
+                    qp2p::SendError::ConnectionLost(qp2p::ConnectionError::Closed(
+                        qp2p::Close::Local,
+                    )) => Error::ConnectionClosed,
                     _ => {
                         trace!("during sending, received error {:?}", err);
-                        Error::AddressNotReachable { err }
+                        err.into()
                     }
                 });
 
@@ -299,12 +301,13 @@ pub(crate) enum SendStatus {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::messaging::{section_info::SectionInfoMsg, DstLocation, WireMsg};
-    use crate::types::PublicKey;
+    use crate::messaging::{DstLocation, MessageId, MsgKind, ServiceAuth};
+    use crate::types::Keypair;
     use assert_matches::assert_matches;
     use eyre::Result;
     use futures::future;
     use qp2p::Config;
+    use rand::rngs::OsRng;
     use std::{net::Ipv4Addr, time::Duration};
     use tokio::{net::UdpSocket, sync::mpsc, time};
 
@@ -313,16 +316,16 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     async fn successful_send() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
-        let comm = Comm::new(transport_config(), tx).await?;
+        let comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
         let mut peer0 = Peer::new().await?;
         let mut peer1 = Peer::new().await?;
 
-        let original_message = new_section_info_message()?;
+        let original_message = new_test_message()?;
 
         let status = comm
             .send(
-                &[(peer0._name, peer0.addr), (peer1._name, peer1.addr)],
+                &[(peer0.name, peer0.addr), (peer1.name, peer1.addr)],
                 2,
                 original_message.clone(),
             )
@@ -344,15 +347,15 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     async fn successful_send_to_subset() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
-        let comm = Comm::new(transport_config(), tx).await?;
+        let comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
         let mut peer0 = Peer::new().await?;
         let mut peer1 = Peer::new().await?;
 
-        let original_message = new_section_info_message()?;
+        let original_message = new_test_message()?;
         let status = comm
             .send(
-                &[(peer0._name, peer0.addr), (peer1._name, peer1.addr)],
+                &[(peer0.name, peer0.addr), (peer1.name, peer1.addr)],
                 1,
                 original_message.clone(),
             )
@@ -376,10 +379,11 @@ mod tests {
     async fn failed_send() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
         let comm = Comm::new(
+            local_addr(),
             Config {
                 // This makes this test faster.
-                idle_timeout_msec: Some(1),
-                ..transport_config()
+                idle_timeout: Some(Duration::from_millis(1)),
+                ..Config::default()
             },
             tx,
         )
@@ -387,11 +391,7 @@ mod tests {
         let invalid_addr = get_invalid_addr().await?;
 
         let status = comm
-            .send(
-                &[(XorName::random(), invalid_addr)],
-                1,
-                new_section_info_message()?,
-            )
+            .send(&[(XorName::random(), invalid_addr)], 1, new_test_message()?)
             .await?;
 
         assert_matches!(
@@ -406,9 +406,10 @@ mod tests {
     async fn successful_send_after_failed_attempts() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
         let comm = Comm::new(
+            local_addr(),
             Config {
-                idle_timeout_msec: Some(1),
-                ..transport_config()
+                idle_timeout: Some(Duration::from_millis(1)),
+                ..Config::default()
             },
             tx,
         )
@@ -417,10 +418,10 @@ mod tests {
         let invalid_addr = get_invalid_addr().await?;
         let name = XorName::random();
 
-        let message = new_section_info_message()?;
+        let message = new_test_message()?;
         let _ = comm
             .send(
-                &[(name, invalid_addr), (peer._name, peer.addr)],
+                &[(name, invalid_addr), (peer.name, peer.addr)],
                 1,
                 message.clone(),
             )
@@ -436,9 +437,10 @@ mod tests {
     async fn partially_successful_send() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
         let comm = Comm::new(
+            local_addr(),
             Config {
-                idle_timeout_msec: Some(1),
-                ..transport_config()
+                idle_timeout: Some(Duration::from_millis(1)),
+                ..Config::default()
             },
             tx,
         )
@@ -447,10 +449,10 @@ mod tests {
         let invalid_addr = get_invalid_addr().await?;
         let name = XorName::random();
 
-        let message = new_section_info_message()?;
+        let message = new_test_message()?;
         let status = comm
             .send(
-                &[(name, invalid_addr), (peer._name, peer.addr)],
+                &[(name, invalid_addr), (peer.name, peer.addr)],
                 2,
                 message.clone(),
             )
@@ -470,21 +472,14 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     async fn send_after_reconnect() -> Result<()> {
         let (tx, _rx) = mpsc::channel(1);
-        let send_comm = Comm::new(transport_config(), tx).await?;
+        let send_comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
-        let recv_transport = QuicP2p::with_config(Some(transport_config()), &[], false)?;
-        let (recv_endpoint, _, mut incoming_msgs, _) = recv_transport.new_endpoint().await?;
-        let recv_addr = recv_endpoint.socket_addr();
+        let (recv_endpoint, _, mut incoming_msgs, _, _) =
+            Endpoint::<XorName>::new(local_addr(), &[], Config::default()).await?;
+        let recv_addr = recv_endpoint.public_addr();
         let name = XorName::random();
 
-        // Send the first message.
-        let key0 = bls::SecretKey::random().public_key();
-        let query = SectionInfoMsg::GetSectionQuery(PublicKey::Bls(key0));
-        let dst_location = DstLocation::Node {
-            name,
-            section_pk: key0,
-        };
-        let msg0 = WireMsg::new_section_info_msg(&query, dst_location)?;
+        let msg0 = new_test_message()?;
         let _ = send_comm
             .send(&[(name, recv_addr)], 1, msg0.clone())
             .await?;
@@ -496,19 +491,12 @@ mod tests {
             if let Some((src, msg)) = time::timeout(TIMEOUT, incoming_msgs.next()).await? {
                 assert_eq!(WireMsg::from(msg)?, msg0);
                 msg0_received = true;
-                recv_endpoint.disconnect_from(&src).await?;
+                recv_endpoint.disconnect_from(&src).await;
             }
             assert!(msg0_received);
         }
 
-        // Send the second message.
-        let key1 = bls::SecretKey::random().public_key();
-        let query = SectionInfoMsg::GetSectionQuery(PublicKey::Bls(key1));
-        let dst_location = DstLocation::Node {
-            name,
-            section_pk: key1,
-        };
-        let msg1 = WireMsg::new_section_info_msg(&query, dst_location)?;
+        let msg1 = new_test_message()?;
         let _ = send_comm
             .send(&[(name, recv_addr)], 1, msg1.clone())
             .await?;
@@ -528,20 +516,16 @@ mod tests {
     #[tokio::test(flavor = "multi_thread")]
     async fn incoming_connection_lost() -> Result<()> {
         let (tx, mut rx0) = mpsc::channel(1);
-        let comm0 = Comm::new(transport_config(), tx).await?;
+        let comm0 = Comm::new(local_addr(), Config::default(), tx).await?;
         let addr0 = comm0.our_connection_info();
 
         let (tx, _rx) = mpsc::channel(1);
-        let comm1 = Comm::new(transport_config(), tx).await?;
+        let comm1 = Comm::new(local_addr(), Config::default(), tx).await?;
         let addr1 = comm1.our_connection_info();
 
         // Send a message to establish the connection
         let _ = comm1
-            .send(
-                &[(XorName::random(), addr0)],
-                1,
-                new_section_info_message()?,
-            )
+            .send(&[(XorName::random(), addr0)], 1, new_test_message()?)
             .await?;
 
         assert_matches!(rx0.recv().await, Some(ConnectionEvent::Received(_)));
@@ -556,40 +540,42 @@ mod tests {
         Ok(())
     }
 
-    fn transport_config() -> Config {
-        Config {
-            local_ip: Some(Ipv4Addr::LOCALHOST.into()),
-            ..Default::default()
-        }
-    }
-
-    fn new_section_info_message() -> Result<WireMsg> {
-        let random_bls_pk = bls::SecretKey::random().public_key();
-        let query = SectionInfoMsg::GetSectionQuery(PublicKey::Bls(random_bls_pk));
+    fn new_test_message() -> Result<WireMsg> {
         let dst_location = DstLocation::Node {
             name: XorName::random(),
             section_pk: bls::SecretKey::random().public_key(),
         };
 
-        let wire_msg = WireMsg::new_section_info_msg(&query, dst_location)?;
+        let mut rng = OsRng;
+        let src_keypair = Keypair::new_ed25519(&mut rng);
+
+        let payload = WireMsg::serialize_msg_payload(&"test_string".to_string())?;
+        let auth = ServiceAuth {
+            public_key: src_keypair.public_key(),
+            signature: src_keypair.sign(&payload),
+        };
+
+        let wire_msg = WireMsg::new_msg(
+            MessageId::new(),
+            payload,
+            MsgKind::ServiceMsg(auth),
+            dst_location,
+        )?;
+
         Ok(wire_msg)
     }
 
     struct Peer {
         addr: SocketAddr,
-        _incoming_connections: qp2p::IncomingConnections,
-        _disconnections: qp2p::DisconnectionEvents,
-        _name: XorName,
+        name: XorName,
         rx: mpsc::Receiver<Bytes>,
     }
 
     impl Peer {
         async fn new() -> Result<Self> {
-            let transport = QuicP2p::with_config(Some(transport_config()), &[], false)?;
-
-            let (endpoint, incoming_connections, mut incoming_messages, disconnections) =
-                transport.new_endpoint().await?;
-            let addr = endpoint.socket_addr();
+            let (endpoint, _, mut incoming_messages, _, _) =
+                Endpoint::<XorName>::new(local_addr(), &[], Config::default()).await?;
+            let addr = endpoint.public_addr();
 
             let (tx, rx) = mpsc::channel(1);
 
@@ -602,9 +588,7 @@ mod tests {
             Ok(Self {
                 addr,
                 rx,
-                _incoming_connections: incoming_connections,
-                _disconnections: disconnections,
-                _name: XorName::random(),
+                name: XorName::random(),
             })
         }
     }
@@ -622,4 +606,8 @@ mod tests {
 
         Ok(addr)
     }
+
+    fn local_addr() -> SocketAddr {
+        (Ipv4Addr::LOCALHOST, 0).into()
+    }
 }
