@@ -8,9 +8,9 @@
 
 use super::Core;
 use crate::messaging::{
-    node::{
-        DkgKey, ElderCandidates, JoinResponse, Network, NodeMsg, NodeState, Peer, Proposal,
-        RelocateDetails, RelocatePromise, Section, SectionAuth,
+    system::{
+        DkgKey, ElderCandidates, JoinResponse, NodeState, Peer, Proposal, RelocateDetails,
+        RelocatePromise, Section, SectionAuth, SystemMsg,
     },
     DstLocation, WireMsg,
 };
@@ -18,17 +18,16 @@ use crate::routing::{
     dkg::{DkgKeyUtils, ProposalUtils, SigShare},
     error::Result,
     messages::WireMsgUtils,
-    network::NetworkUtils,
     peer::PeerUtils,
     relocation::RelocateState,
     routing_api::command::Command,
-    section::{ElderCandidatesUtils, SectionKeyShare, SectionUtils},
+    section::{SectionKeyShare, SectionUtils},
     SectionAuthorityProviderUtils,
 };
+use crate::types::PublicKey;
 use bls::PublicKey as BlsPublicKey;
 use std::{net::SocketAddr, slice};
 use xor_name::XorName;
-
 impl Core {
     // Send proposal to all our elders.
     pub(crate) fn propose(&self, proposal: Proposal) -> Result<Vec<Command>> {
@@ -69,13 +68,17 @@ impl Core {
         )?;
 
         // Broadcast the proposal to the rest of the section elders.
-        let node_msg = NodeMsg::Propose {
+        let node_msg = SystemMsg::Propose {
             content: proposal,
             sig_share,
         };
+        let section_pk = *self.section.chain().last_key();
         let wire_msg = WireMsg::single_src(
             &self.node,
-            DstLocation::DirectAndUnrouted(*self.section.chain().last_key()),
+            DstLocation::Section {
+                name: XorName::from(PublicKey::Bls(section_pk)),
+                section_pk,
+            },
             node_msg,
             self.section.authority_provider().section_key(),
         )?;
@@ -96,17 +99,11 @@ impl Core {
         if self.section.chain().has_key(&public_key)
             && public_key != *self.section.chain().last_key()
         {
-            // The key is recognized as non-last, indicating the peer is lagging.
-            Ok(Some(self.send_direct_message(
-                peer,
-                // TODO: consider sending only those parts of section that are new
-                // since `public_key` was the latest key.
-                NodeMsg::Sync {
-                    section: self.section.clone(),
-                    network: self.network.clone(),
-                },
-                sig_share.public_key_set.public_key(),
-            )?))
+            let dst_section_pk = sig_share.public_key_set.public_key();
+            let msg = self.generate_ae_update(dst_section_pk, true)?;
+
+            let cmd = self.send_direct_message(peer, msg, dst_section_pk)?;
+            Ok(Some(cmd))
         } else {
             Ok(None)
         }
@@ -123,7 +120,7 @@ impl Core {
         let addr = *node_state.value.peer.addr();
         let name = *node_state.value.peer.name();
 
-        let node_msg = NodeMsg::JoinResponse(Box::new(JoinResponse::Approval {
+        let node_msg = SystemMsg::JoinResponse(Box::new(JoinResponse::Approval {
             genesis_key: *self.section.genesis_key(),
             section_auth: self.section.section_signed_authority_provider().clone(),
             node_state,
@@ -136,47 +133,36 @@ impl Core {
         Ok(cmd)
     }
 
-    pub(crate) fn send_sync(&mut self, section: Section, network: Network) -> Result<Vec<Command>> {
-        let (elders, non_elders): (Vec<_>, _) = section
+    pub(crate) fn send_ae_update_to_our_section(
+        &mut self,
+        section: Section,
+    ) -> Result<Vec<Command>> {
+        let nodes: Vec<_> = section
             .active_members()
             .filter(|peer| peer.name() != &self.node.name())
             .map(|peer| (*peer.name(), *peer.addr()))
-            .partition(|peer| section.is_elder(&peer.0));
+            .collect();
 
-        // Send the trimmed state to non-elders. The trimmed state contains only the knowledge of
-        // own section.
+        // the PK is that of our section (as we know it; and we're ahead of our adults here)
         let dst_section_pk = *self.section_chain().last_key();
-        let node_msg = NodeMsg::Sync {
-            section: section.clone(),
-            network: Network::new(),
-        };
+        // the previous PK which is likely what adults know
+        let previous_pk = *self.section_chain().prev_key();
+        let node_msg = self.generate_ae_update(previous_pk, true)?;
+        let cmd = self.send_direct_message_to_nodes(nodes, node_msg, dst_section_pk)?;
 
-        let cmd = self.send_direct_message_to_nodes(non_elders, node_msg, dst_section_pk)?;
-
-        let mut commands = vec![cmd];
-
-        // Send the full state to elders.
-        // The full state contains the whole section chain.
-        let node_msg = NodeMsg::Sync { section, network };
-        let cmd = self.send_direct_message_to_nodes(elders, node_msg, dst_section_pk)?;
-        commands.push(cmd);
-
-        Ok(commands)
+        Ok(vec![cmd])
     }
 
-    pub(crate) fn send_sync_to_adults(&mut self) -> Result<Vec<Command>> {
+    pub(crate) fn send_ae_update_to_adults(&mut self) -> Result<Vec<Command>> {
         let adults: Vec<_> = self
             .section
             .live_adults()
             .map(|peer| (*peer.name(), *peer.addr()))
             .collect();
 
-        let node_msg = NodeMsg::Sync {
-            section: self.section.clone(),
-            network: Network::new(),
-        };
-
         let dst_section_pk = *self.section_chain().last_key();
+        let node_msg = self.generate_ae_update(dst_section_pk, true)?;
+
         let cmd = self.send_direct_message_to_nodes(adults, node_msg, dst_section_pk)?;
 
         Ok(vec![cmd])
@@ -192,7 +178,7 @@ impl Core {
             name: details.pub_id,
             section_pk: *self.section.chain().last_key(),
         };
-        let node_msg = NodeMsg::Relocate(details);
+        let node_msg = SystemMsg::Relocate(details);
 
         self.send_message_for_dst_accumulation(src, dst, node_msg, slice::from_ref(recipient))
     }
@@ -210,7 +196,7 @@ impl Core {
             name: promise.name,
             section_pk: *self.section.chain().last_key(),
         };
-        let node_msg = NodeMsg::RelocatePromise(promise);
+        let node_msg = SystemMsg::RelocatePromise(promise);
 
         self.send_message_for_dst_accumulation(src, dst, node_msg, slice::from_ref(recipient))
     }
@@ -224,13 +210,7 @@ impl Core {
         }
     }
 
-    pub(crate) fn send_dkg_start(&self, elder_candidates: ElderCandidates) -> Result<Vec<Command>> {
-        // Send to all participants.
-        let recipients: Vec<_> = elder_candidates.peers().collect();
-        self.send_dkg_start_to(elder_candidates, &recipients)
-    }
-
-    pub(crate) fn send_dkg_start_to(
+    pub(crate) fn send_dkg_start(
         &self,
         elder_candidates: ElderCandidates,
         recipients: &[Peer],
@@ -246,14 +226,17 @@ impl Core {
             recipients
         );
 
-        let node_msg = NodeMsg::DkgStart {
+        let node_msg = SystemMsg::DkgStart {
             dkg_key,
             elder_candidates,
         };
-
+        let section_pk = *self.section.chain().last_key();
         self.send_message_for_dst_accumulation(
             src_prefix.name(),
-            DstLocation::DirectAndUnrouted(*self.section.chain().last_key()),
+            DstLocation::Section {
+                name: XorName::from(PublicKey::Bls(section_pk)),
+                section_pk,
+            },
             node_msg,
             recipients,
         )
@@ -263,7 +246,7 @@ impl Core {
         &self,
         src: XorName,
         dst: DstLocation,
-        node_msg: NodeMsg,
+        node_msg: SystemMsg,
         recipients: &[Peer],
     ) -> Result<Vec<Command>> {
         let key_share = self.section_keys_provider.key_share().map_err(|err| {
@@ -339,12 +322,15 @@ impl Core {
     pub(crate) fn send_direct_message(
         &self,
         recipient: (XorName, SocketAddr),
-        node_msg: NodeMsg,
+        node_msg: SystemMsg,
         dst_section_pk: BlsPublicKey,
     ) -> Result<Command> {
         let wire_msg = WireMsg::single_src(
             &self.node,
-            DstLocation::DirectAndUnrouted(dst_section_pk),
+            DstLocation::Section {
+                name: recipient.0,
+                section_pk: dst_section_pk,
+            },
             node_msg,
             self.section.authority_provider().section_key(),
         )?;
@@ -358,12 +344,15 @@ impl Core {
     pub(crate) fn send_direct_message_to_nodes(
         &self,
         recipients: Vec<(XorName, SocketAddr)>,
-        node_msg: NodeMsg,
+        node_msg: SystemMsg,
         dst_section_pk: BlsPublicKey,
     ) -> Result<Command> {
         let wire_msg = WireMsg::single_src(
             &self.node,
-            DstLocation::DirectAndUnrouted(dst_section_pk),
+            DstLocation::Section {
+                name: XorName::from(PublicKey::Bls(dst_section_pk)),
+                section_pk: dst_section_pk,
+            },
             node_msg,
             self.section.authority_provider().section_key(),
         )?;
@@ -376,7 +365,7 @@ impl Core {
 
     // TODO: consider changing this so it sends only to a subset of the elders
     // (say 1/3 of the ones closest to our name or so)
-    pub(crate) fn send_message_to_our_elders(&self, node_msg: NodeMsg) -> Result<Command> {
+    pub(crate) fn send_message_to_our_elders(&self, node_msg: SystemMsg) -> Result<Command> {
         let targets: Vec<_> = self
             .section
             .authority_provider()
