@@ -23,10 +23,10 @@
 // ############################################################################
 // Public API
 // ############################################################################
-pub use self::error::AggregatorError;
 pub use self::error::ProposalError;
 pub use self::{
     cache::Cache,
+    dkg::SectionAuthUtils,
     error::{Error, Result},
     peer::PeerUtils,
     routing_api::{
@@ -40,7 +40,13 @@ pub use self::{
         section_authority_provider::SectionAuthorityProviderUtils,
     },
 };
-pub(crate) use self::{core::SignatureAggregator, section::section_keys::SectionKeyShare};
+pub(crate) use self::{
+    core::ChunkStore,
+    core::RegisterStorage,
+    core::{CHUNK_COPY_COUNT, MIN_LEVEL_WHEN_FULL},
+    section::section_keys::SectionKeyShare,
+};
+
 pub use qp2p::{Config as TransportConfig, SendStream};
 
 pub use xor_name::{Prefix, XorName, XOR_NAME_LEN}; // TODO remove pub on API update
@@ -48,6 +54,11 @@ pub use xor_name::{Prefix, XorName, XOR_NAME_LEN}; // TODO remove pub on API upd
 #[cfg(any(test, feature = "test-utils"))]
 pub use test_utils::*;
 
+#[cfg(test)]
+pub(crate) use dkg::test_utils::section_signed;
+#[cfg(test)]
+pub(crate) use section::test_utils::gen_section_authority_provider;
+
 // ############################################################################
 // Private
 // ############################################################################
@@ -58,7 +69,6 @@ mod dkg;
 mod ed25519;
 mod error;
 mod messages;
-mod network;
 mod node;
 mod peer;
 mod relocation;
