@@ -23,26 +23,25 @@ use self::{
     event::{Elders, Event, NodeElderChange},
     event_stream::EventStream,
 };
-use crate::dbs::UsedSpace;
+use crate::messaging::{
+    data::StorageLevel,
+    system::{Peer, SystemMsg},
+    DstLocation, SectionAuthorityProvider, WireMsg,
+};
 use crate::routing::{
-    core::{join_network, Comm, ConnectionEvent, Core},
+    core::{join_network, ChunkStore, Comm, ConnectionEvent, Core, RegisterStorage},
     ed25519,
     error::Result,
     messages::WireMsgUtils,
-    network::NetworkUtils,
     node::Node,
     peer::PeerUtils,
     section::SectionUtils,
-    Error, SectionAuthorityProviderUtils, MIN_ADULT_AGE,
-};
-use crate::{
-    messaging::{
-        node::{NodeMsg, Peer},
-        DstLocation, SectionAuthorityProvider, WireMsg,
-    },
-    node::RegisterStorage,
+    SectionAuthorityProviderUtils, MIN_ADULT_AGE,
 };
+use crate::{dbs::UsedSpace, messaging::data::ChunkDataExchange};
 use ed25519_dalek::{PublicKey, Signature, Signer, KEYPAIR_LENGTH};
+
+use crate::types::PublicKey as DtPublicKey;
 use itertools::Itertools;
 use secured_linked_list::SecuredLinkedList;
 use std::path::PathBuf;
@@ -93,7 +92,12 @@ impl Routing {
                 std::process::id()
             );
 
-            let comm = Comm::new(config.transport_config, connection_event_tx).await?;
+            let comm = Comm::new(
+                config.local_addr,
+                config.transport_config,
+                connection_event_tx,
+            )
+            .await?;
             let node = Node::new(keypair, comm.our_connection_info());
             let core = Core::first_node(comm, node, event_tx, used_space, root_storage_dir)?;
 
@@ -122,8 +126,13 @@ impl Routing {
             let node_name = ed25519::name(&keypair.public);
             info!("{} Bootstrapping a new node.", node_name);
 
-            let (comm, bootstrap_addr) =
-                Comm::bootstrap(config.transport_config, connection_event_tx).await?;
+            let (comm, bootstrap_addr) = Comm::bootstrap(
+                config.local_addr,
+                &config.bootstrap_nodes,
+                config.transport_config,
+                connection_event_tx,
+            )
+            .await?;
             info!(
                 "{} Joining as a new node (PID: {}) from {} to {}",
                 node_name,
@@ -171,22 +180,40 @@ impl Routing {
         self.dispatcher.get_register_storage().await
     }
 
+    pub(crate) async fn get_chunk_storage(&self) -> ChunkStore {
+        self.dispatcher.get_chunk_storage().await
+    }
+    pub(crate) async fn get_chunk_data_of(&self, prefix: &Prefix) -> ChunkDataExchange {
+        self.dispatcher.get_chunk_data_of(prefix).await
+    }
+
+    /// Returns whether the level changed or not.
+    pub(crate) async fn set_storage_level(
+        &self,
+        node_id: &DtPublicKey,
+        level: StorageLevel,
+    ) -> bool {
+        self.dispatcher.set_storage_level(node_id, level).await
+    }
+    pub(crate) async fn retain_members_only(&self, members: BTreeSet<XorName>) -> Result<()> {
+        self.dispatcher.retain_members_only(members).await
+    }
+
+    pub(crate) async fn update_chunks(&self, chunks: ChunkDataExchange) {
+        self.dispatcher
+            .core
+            .read()
+            .await
+            .update_chunks(chunks)
+            .await
+    }
+
     /// Sets the JoinsAllowed flag.
     pub async fn set_joins_allowed(&self, joins_allowed: bool) -> Result<()> {
         let command = Command::SetJoinsAllowed(joins_allowed);
         self.dispatcher.clone().handle_commands(command).await
     }
 
-    /// Starts a proposal that a node has gone offline.
-    /// This can be done only by an Elder.
-    pub async fn propose_offline(&self, name: XorName) -> Result<()> {
-        if !self.is_elder().await {
-            return Err(Error::InvalidState);
-        }
-        let command = Command::ProposeOffline(name);
-        self.dispatcher.clone().handle_commands(command).await
-    }
-
     /// Signals the Elders of our section to test connectivity to a node.
     pub async fn start_connectivity_test(&self, name: XorName) -> Result<()> {
         let command = Command::StartConnectivityTest(name);
@@ -323,28 +350,15 @@ impl Routing {
             .clone()
     }
 
-    /// Returns the info about other sections in the network known to us.
-    pub async fn other_sections(&self) -> Vec<SectionAuthorityProvider> {
-        self.dispatcher
-            .core
-            .read()
-            .await
-            .network()
-            .all()
-            .cloned()
-            .collect()
-    }
-
     /// Returns the info about the section matching the name.
     pub async fn matching_section(&self, name: &XorName) -> Result<SectionAuthorityProvider> {
-        let core = self.dispatcher.core.read().await;
-        core.matching_section(name)
+        self.dispatcher.core.read().await.matching_section(name)
     }
 
     /// Builds a WireMsg signed by this Node
     pub async fn sign_single_src_msg(
         &self,
-        node_msg: NodeMsg,
+        node_msg: SystemMsg,
         dst: DstLocation,
     ) -> Result<WireMsg> {
         let src_section_pk = *self.section_chain().await.last_key();
@@ -359,7 +373,7 @@ impl Routing {
     /// Builds a WireMsg signed for accumulateion at destination
     pub async fn sign_msg_for_dst_accumulation(
         &self,
-        node_msg: NodeMsg,
+        node_msg: SystemMsg,
         dst: DstLocation,
     ) -> Result<WireMsg> {
         let src = self.name().await;
